@@ -2,7 +2,7 @@
  *
  * This file is part of PRoot.
  *
- * Copyright (C) 2014 STMicroelectronics
+ * Copyright (C) 2015 STMicroelectronics
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -36,6 +36,7 @@
 #include "syscall/syscall.h"
 #include "syscall/sysnum.h"
 #include "syscall/seccomp.h"
+#include "execve/execve.h"
 #include "tracee/tracee.h"
 #include "tracee/abi.h"
 #include "tracee/mem.h"
@@ -124,6 +125,7 @@ static FilteredSysnum filtered_sysnums[] = {
 	{ PR_lsetxattr,		FILTER_SYSEXIT },
 	{ PR_fsetxattr,		FILTER_SYSEXIT },
 	{ PR_stat,		FILTER_SYSEXIT },
+	{ PR_statx,		FILTER_SYSEXIT },
 	{ PR_stat64,		FILTER_SYSEXIT },
 	{ PR_statfs,		FILTER_SYSEXIT },
 	{ PR_statfs64,		FILTER_SYSEXIT },
@@ -200,6 +202,7 @@ static void override_permissions(const Tracee *tracee, const char *path, bool is
 		case PR_oldlstat:
 		case PR_oldstat:
 		case PR_stat:
+		case PR_statx:
 		case PR_stat64:
 		case PR_statfs:
 		case PR_statfs64:
@@ -312,7 +315,7 @@ static int handle_sysenter_end(Tracee *tracee, const Config *config)
  * Copy config->@field to the tracee's memory location pointed to by @sysarg.
  */
 #define POKE_MEM_ID(sysarg, field) do {					\
-	poke_uint16(tracee, peek_reg(tracee, ORIGINAL, sysarg), config->field);	\
+	poke_uint32(tracee, peek_reg(tracee, ORIGINAL, sysarg), config->field);	\
 	if (errno != 0)							\
 		return -errno;						\
 } while (0)
@@ -617,12 +620,15 @@ static int handle_sysexit_end(Tracee *tracee, Config *config)
 	case PR_lstat64:
 	case PR_fstat64:
 	case PR_stat:
+	case PR_statx:
 	case PR_lstat:
 	case PR_fstat: {
 		word_t address;
 		Reg sysarg;
 		uid_t uid;
 		gid_t gid;
+		off_t uid_offset;
+		off_t gid_offset;
 
 		/* Override only if it succeed.  */
 		result = peek_reg(tracee, CURRENT, SYSARG_RESULT);
@@ -630,10 +636,19 @@ static int handle_sysexit_end(Tracee *tracee, Config *config)
 			return 0;
 
 		/* Get the address of the 'stat' structure.  */
-		if (sysnum == PR_fstatat64 || sysnum == PR_newfstatat)
-			sysarg = SYSARG_3;
-		else
-			sysarg = SYSARG_2;
+		if (sysnum == PR_statx) {
+			sysarg = SYSARG_5;
+			uid_offset = OFFSETOF_STATX_UID;
+			gid_offset = OFFSETOF_STATX_GID;
+		}
+		else {
+			if (sysnum == PR_fstatat64 || sysnum == PR_newfstatat)
+				sysarg = SYSARG_3;
+			else
+				sysarg = SYSARG_2;
+			uid_offset = offsetof_stat_uid(tracee);
+			gid_offset = offsetof_stat_gid(tracee);
+		}
 
 		address = peek_reg(tracee, ORIGINAL, sysarg);
 
@@ -642,27 +657,28 @@ static int handle_sysexit_end(Tracee *tracee, Config *config)
 		assert(__builtin_types_compatible_p(gid_t, uint32_t));
 
 		/* Get the uid & gid values from the 'stat' structure.  */
-		uid = peek_uint32(tracee, address + offsetof_stat_uid(tracee));
+		uid = peek_uint32(tracee, address + uid_offset);
 		if (errno != 0)
 			uid = 0; /* Not fatal.  */
 
-		gid = peek_uint32(tracee, address + offsetof_stat_gid(tracee));
+		gid = peek_uint32(tracee, address + gid_offset);
 		if (errno != 0)
 			gid = 0; /* Not fatal.  */
 
 		/* Override only if the file is owned by the current user.
 		 * Errors are not fatal here.  */
 		if (uid == getuid())
-			poke_uint32(tracee, address + offsetof_stat_uid(tracee), config->ruid);
+			poke_uint32(tracee, address + uid_offset, config->suid);
 
 		if (gid == getgid())
-			poke_uint32(tracee, address + offsetof_stat_gid(tracee), config->rgid);
+			poke_uint32(tracee, address + gid_offset, config->sgid);
 
 		return 0;
 	}
 
 	case PR_chroot: {
 		char path[PATH_MAX];
+		char abspath[PATH_MAX];
 		word_t input;
 		int status;
 
@@ -680,8 +696,12 @@ static int handle_sysexit_end(Tracee *tracee, Config *config)
 		if (status < 0)
 			return status;
 
+		/* Resolve relative path segments. */
+		if (!realpath(path, abspath))
+			return 0;
+
 		/* Only "new rootfs == current rootfs" is supported yet.  */
-		status = compare_paths(get_root(tracee), path);
+		status = compare_paths(get_root(tracee), abspath);
 		if (status != PATHS_ARE_EQUAL)
 			return 0;
 
@@ -852,13 +872,32 @@ int fake_id0_callback(Extension *extension, ExtensionEvent event, intptr_t data1
 	case SYSCALL_EXIT_START: {
 		Tracee *tracee = TRACEE(extension);
 		Config *config = talloc_get_type_abort(extension->config, Config);
-		word_t result = peek_reg(tracee, CURRENT, SYSARG_RESULT);;
+		word_t result = peek_reg(tracee, CURRENT, SYSARG_RESULT);
 		word_t sysnum = get_sysnum(tracee, ORIGINAL);
+		struct stat mode;
+		int status;
+
+		if ((int) result < 0 || sysnum != PR_execve)
+			return 0;
+
+		/* This has to be done before PRoot pushes the load
+		 * script into tracee's stack.  */
+		adjust_elf_auxv(tracee, config);
+
+		status = stat(tracee->load_info->host_path, &mode);
+		if (status < 0)
+			return 0; /* Not fatal.  */
+
+		if ((mode.st_mode & S_ISUID) != 0) {
+			config->euid = 0;
+			config->suid = 0;
+		}
+
+		if ((mode.st_mode & S_ISGID) != 0) {
+			config->egid = 0;
+			config->sgid = 0;
+		}
 
-		/* Note: this can be done only before PRoot pushes the
-		 * load script into tracee's stack.  */
-		if ((int) result >= 0 && sysnum == PR_execve)
-			adjust_elf_auxv(tracee, config);
 		return 0;
 	}
 
