@@ -2,7 +2,7 @@
  *
  * This file is part of PRoot.
  *
- * Copyright (C) 2014 STMicroelectronics
+ * Copyright (C) 2015 STMicroelectronics
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -313,10 +313,10 @@ static int handle_sysenter_end(Tracee *tracee, Config *config)
 			}
 		};
 
-		flags = peek_reg(tracee, CURRENT, SYSARG_4);
-		if ((flags & ~AT_SYMLINK_NOFOLLOW) != 0)
-			return -EINVAL; /* Exposed by LTP.  */
+		if (config->actual_release == 0)
+			return 0;
 
+		flags = peek_reg(tracee, CURRENT, SYSARG_4);
 #if defined(ARCH_X86_64)
 		if ((flags & AT_SYMLINK_NOFOLLOW) != 0)
 			modif.new_sysarg_num = (get_abi(tracee) != ABI_2 ? PR_lstat : PR_lstat64);
@@ -329,7 +329,14 @@ static int handle_sysenter_end(Tracee *tracee, Config *config)
 			modif.new_sysarg_num = PR_stat64;
 #endif
 
-		modify_syscall(tracee, config, &modif);
+		if (modify_syscall(tracee, config, &modif)) {
+			// Do this check only if we are patching this syscall.
+			// New flags have been added since 2.6.38
+			// that we should not error on.
+			if ((flags & ~AT_SYMLINK_NOFOLLOW) != 0) {
+				return -EINVAL; /* Exposed by LTP.  */
+			}
+		}
 		return 0;
 	}
 
@@ -509,6 +516,24 @@ static int handle_sysenter_end(Tracee *tracee, Config *config)
 		return 0;
 	}
 
+	case PR_renameat2: {
+		Modif modif = {
+			.expected_release = KERNEL_VERSION(3,15,0),
+			.new_sysarg_num   = PR_rename,
+			.shifts = { [0] = {
+					.sysarg  = SYSARG_2,
+					.nb_args = 1,
+					.offset  =-1 },
+				    [1] = {
+					    .sysarg  = SYSARG_4,
+					    .nb_args = 1,
+					    .offset  = -2 }
+			}
+		};
+		modify_syscall(tracee, config, &modif);
+		return 0;
+	}
+
 	case PR_signalfd4: {
 		bool modified;
 		Modif modif = {
@@ -587,6 +612,7 @@ static void adjust_elf_auxv(Tracee *tracee, Config *config)
 	word_t stack_pointer;
 	void *argv_envp;
 	size_t size;
+	size_t reserve_size;
 	int status;
 
 	vectors_address = get_elf_aux_vectors_address(tracee);
@@ -653,8 +679,11 @@ static void adjust_elf_auxv(Tracee *tracee, Config *config)
 
 	/* Allocate enough room in tracee's stack for the new ELF
 	 * auxiliary vector.  */
-	stack_pointer   -= 2 * sizeof_word(tracee);
-	vectors_address -= 2 * sizeof_word(tracee);
+	reserve_size = 2 * sizeof_word(tracee);
+	/* Make sure the stack is still aligned */
+	reserve_size = ((reserve_size - 1) / STACK_ALIGNMENT + 1) * STACK_ALIGNMENT;
+	stack_pointer   -= reserve_size;
+	vectors_address -= reserve_size;
 
 	/* Note that it is safe to update the stack pointer manually
 	 * since we are in execve sysexit.  However it should be done
@@ -968,6 +997,7 @@ static FilteredSysnum filtered_sysnums[] = {
 	{ PR_pselect6, 		0 },
 	{ PR_readlinkat, 	0 },
 	{ PR_renameat, 		0 },
+	{ PR_renameat2,		0 },
 	{ PR_setdomainname,	FILTER_SYSEXIT },
 	{ PR_sethostname,	FILTER_SYSEXIT },
 	{ PR_signalfd4, 	FILTER_SYSEXIT },
