@@ -2,7 +2,7 @@
  *
  * This file is part of PRoot.
  *
- * Copyright (C) 2014 STMicroelectronics
+ * Copyright (C) 2015 STMicroelectronics
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -129,10 +129,9 @@ static inline Finality next_component(char component[NAME_MAX], const char **cur
  * Resolve bindings (if any) in @guest_path and copy the translated
  * path into @host_path.  Also, this function checks that a non-final
  * component is either a directory (returned value is 0) or a symlink
- * (returned value is 1), otherwise it returns -errno (-ENOENT or
- * -ENOTDIR).
+ * (returned value is 1), otherwise it returns -errno or -ENOTDIR.
  */
-static inline int substitute_binding_stat(Tracee *tracee, Finality finality,
+static inline int substitute_binding_stat(Tracee *tracee, Finality finality, unsigned int recursion_level,
 					const char guest_path[PATH_MAX], char host_path[PATH_MAX])
 {
 	struct stat statl;
@@ -145,7 +144,8 @@ static inline int substitute_binding_stat(Tracee *tracee, Finality finality,
 
 	/* Don't notify extensions during the initialization of a binding.  */
 	if (tracee->glue_type == 0) {
-		status = notify_extensions(tracee, HOST_PATH, (intptr_t)host_path, finality);
+		status = notify_extensions(tracee, HOST_PATH, (intptr_t)host_path,
+					IS_FINAL(finality) && recursion_level == 0);
 		if (status < 0)
 			return status;
 	}
@@ -161,12 +161,12 @@ static inline int substitute_binding_stat(Tracee *tracee, Finality finality,
 			status = -1;
 	}
 
-	/* Return an error if a non-final component isn't a
-	 * directory nor a symlink.  The error is "No such
-	 * file or directory" if this component doesn't exist,
-	 * otherwise the error is "Not a directory".  */
+	/* Return an error if a non-final component isn't a directory
+	 * nor a symlink.  The error depends on why the component
+	 * could not be accessed (ENOENT, EACCES, ...), otherwise the
+	 * error is "Not a directory".  */
 	if (!IS_FINAL(finality) && !S_ISDIR(statl.st_mode) && !S_ISLNK(statl.st_mode))
-		return (status < 0 ? -ENOENT : -ENOTDIR);
+		return (status < 0 ? -errno : -ENOTDIR);
 
 	return (S_ISLNK(statl.st_mode) ? 1 : 0);
 }
@@ -185,6 +185,7 @@ int canonicalize(Tracee *tracee, const char *user_path, bool deref_final,
 		 char guest_path[PATH_MAX], unsigned int recursion_level)
 {
 	char scratch_path[PATH_MAX];
+	char host_path[PATH_MAX];
 	Finality finality;
 	const char *cursor;
 	int status;
@@ -210,13 +211,20 @@ int canonicalize(Tracee *tracee, const char *user_path, bool deref_final,
 	else
 		strcpy(guest_path, "/");
 
+
+	/* Resolve bindings for the initial '/' component or user_path,
+	 * which is not handled in the loop below.
+	 * In particular HOST_PATH extensions are called from there.  */
+	status = substitute_binding_stat(tracee, NOT_FINAL, recursion_level, guest_path, host_path);
+	if (status < 0)
+		return status;
+
 	/* Canonicalize recursely 'user_path' into 'guest_path'.  */
 	cursor = user_path;
 	finality = NOT_FINAL;
 	while (!IS_FINAL(finality)) {
 		Comparison comparison;
 		char component[NAME_MAX];
-		char host_path[PATH_MAX];
 
 		finality = next_component(component, &cursor);
 		status = (int) finality;
@@ -245,7 +253,7 @@ int canonicalize(Tracee *tracee, const char *user_path, bool deref_final,
 		 * symlink.  For this latter case, we check that the
 		 * symlink points to a directory once it is
 		 * canonicalized, at the end of this loop.  */
-		status = substitute_binding_stat(tracee, finality, scratch_path, host_path);
+		status = substitute_binding_stat(tracee, finality, recursion_level, scratch_path, host_path);
 		if (status < 0)
 			return status;
 
@@ -323,7 +331,7 @@ int canonicalize(Tracee *tracee, const char *user_path, bool deref_final,
 
 		/* Check that a non-final canonicalized/dereferenced
 		 * symlink exists and is a directory.  */
-		status = substitute_binding_stat(tracee, finality, guest_path, host_path);
+		status = substitute_binding_stat(tracee, finality, recursion_level, guest_path, host_path);
 		if (status < 0)
 			return status;
 
