@@ -258,6 +258,13 @@ int translate_ptrace_exit(Tracee *tracee)
 		break;  /* Restart the ptracee.  */
 
 	case PTRACE_SETOPTIONS:
+		if (data & PTRACE_O_TRACESECCOMP) {
+			/* We don't really support forwarding seccomp traps */
+			note(ptracer, WARNING, INTERNAL,
+			     "ptrace option PTRACE_O_TRACESECCOMP "
+			     "not supported yet");
+			return -EINVAL;
+		}
 		PTRACEE.options = data;
 		return 0;  /* Don't restart the ptracee.  */
 
