@@ -2,7 +2,7 @@
  *
  * This file is part of PRoot.
  *
- * Copyright (C) 2014 STMicroelectronics
+ * Copyright (C) 2015 STMicroelectronics
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -30,6 +30,7 @@
 #include "tracee/reg.h"
 #include "tracee/mem.h"
 #include "syscall/sysnum.h"
+#include "execve/execve.h"
 #include "cli/note.h"
 
 #include "compat.h"
@@ -41,27 +42,23 @@
  * mapping is discarded in order to emulate an empty heap.  */
 static word_t heap_offset = 0;
 
-#define PREALLOCATED_HEAP_SIZE (16 * 1024 * 1024)
-
 /**
- * Emulate the brk(2) syscall with mmap(2) and/or mremap(2); this is
- * required to ensure a minimal heap size [1].  This function always
- * returns 0.
- *
- * [1] With some versions of the Linux kernel and some versions of the
- *     GNU ELF interpreter (ld.so), the heap can't grow at all because
- *     the kernel has put a memory mapping right after the heap.  This
- *     issue can be reproduced when using a Ubuntu 12.04 x86_64 rootfs
- *     on RHEL 5 x86_64.
+ * Put @tracee's heap to a reliable location.  By default the Linux
+ * kernel puts it near loader's BSS, but this default location is not
+ * reliable since the kernel might put another memory mapping right
+ * after it (ie. continuously).  In this case, @tracee's heap can't
+ * grow anymore and some programs like Bash will abort.  This issue
+ * can be reproduced when using a Ubuntu 12.04 x86_64 rootfs on RHEL 5
+ * x86_64.
  */
-word_t translate_brk_enter(Tracee *tracee)
+void translate_brk_enter(Tracee *tracee)
 {
 	word_t new_brk_address;
 	size_t old_heap_size;
 	size_t new_heap_size;
 
 	if (tracee->heap->disabled)
-		return 0;
+		return;
 
 	if (heap_offset == 0) {
 		heap_offset = sysconf(_SC_PAGE_SIZE);
@@ -69,18 +66,14 @@ word_t translate_brk_enter(Tracee *tracee)
 			heap_offset = 0x1000;
 	}
 
-	/* Non-fixed mmap pages might be placed right after the
-	 * emulated heap on some architectures.  A solution is to
-	 * preallocate some space to ensure a minimal heap size.  */
-	if (tracee->heap->prealloc_size == 0)
-		tracee->heap->prealloc_size = MAX(PREALLOCATED_HEAP_SIZE, heap_offset);
-
 	new_brk_address = peek_reg(tracee, CURRENT, SYSARG_1);
 	DEBUG_BRK("brk(0x%lx)\n", new_brk_address);
 
 	/* Allocate a new mapping for the emulated heap.  */
 	if (tracee->heap->base == 0) {
 		Sysnum sysnum;
+		Mapping *mappings;
+		Mapping *bss;
 
 		/* From PRoot's point-of-view this is the first time this
 		 * tracee calls brk(2), although an address was specified.
@@ -95,9 +88,24 @@ word_t translate_brk_enter(Tracee *tracee)
 			if (tracee->verbose > 0)
 				note(tracee, WARNING, INTERNAL,
 					"process %d is doing suspicious brk()",	tracee->pid);
-			return 0;
+			return;
 		}
 
+		/* Put the heap as close to the BSS as possible since
+		 * some programs -- like dump-emacs -- assume the gap
+		 * between the end of the BSS and the start of the
+		 * heap is relatively small (ie. < 1MB) even if ALSR
+		 * is enabled.  Note that bss->addr + bss->length is
+		 * naturally aligned to a page boundary according to
+		 * add_mapping() in execve/enter.c, ie. no need to
+		 * align new_brk_address again.  Now, the gap between
+		 * the BSS and the heap is only "heap_offset" bytes
+		 * long.  To emulate ADDR_NO_RANDOMIZE personality,
+		 * this gap should be removed (not yet supported).  */
+		mappings = tracee->load_info->mappings;
+		bss = &mappings[talloc_array_length(mappings) - 1];
+		new_brk_address = bss->addr + bss->length;
+
 		/* I don't understand yet why mmap(2) fails (EFAULT)
 		 * on architectures that also have mmap2(2).  Maybe
 		 * this former implies MAP_FIXED in such cases.  */
@@ -106,46 +114,25 @@ word_t translate_brk_enter(Tracee *tracee)
 			: PR_mmap;
 
 		set_sysnum(tracee, sysnum);
-		poke_reg(tracee, SYSARG_1 /* address */, 0);
-		poke_reg(tracee, SYSARG_2 /* length  */, heap_offset + tracee->heap->prealloc_size);
+		poke_reg(tracee, SYSARG_1 /* address */, new_brk_address);
+		poke_reg(tracee, SYSARG_2 /* length  */, heap_offset);
 		poke_reg(tracee, SYSARG_3 /* prot    */, PROT_READ | PROT_WRITE);
 		poke_reg(tracee, SYSARG_4 /* flags   */, MAP_PRIVATE | MAP_ANONYMOUS);
 		poke_reg(tracee, SYSARG_5 /* fd      */, -1);
 		poke_reg(tracee, SYSARG_6 /* offset  */, 0);
 
-		return 0;
+		return;
 	}
 
 	/* The size of the heap can't be negative.  */
 	if (new_brk_address < tracee->heap->base) {
 		set_sysnum(tracee, PR_void);
-		return 0;
+		return;
 	}
 
 	new_heap_size = new_brk_address - tracee->heap->base;
 	old_heap_size = tracee->heap->size;
 
-	/* Clear the released memory in preallocated space, so it will be
-	 * in the expected state next time it will be reallocated.  */
-	if (new_heap_size < old_heap_size && new_heap_size < tracee->heap->prealloc_size) {
-		(void) clear_mem(tracee,
-				tracee->heap->base + new_heap_size,
-				MIN(old_heap_size, tracee->heap->prealloc_size) - new_heap_size);
-	}
-
-	/* No need to use mremap when both old size and new size are
-	 * in the preallocated space.  */
-	if (   new_heap_size <= tracee->heap->prealloc_size
-	    && old_heap_size <= tracee->heap->prealloc_size) {
-		tracee->heap->size = new_heap_size;
-		set_sysnum(tracee, PR_void);
-		return 0;
-	}
-
-	/* Ensure the preallocated space will never be released.  */
-	new_heap_size = MAX(new_heap_size, tracee->heap->prealloc_size);
-	old_heap_size = MAX(old_heap_size, tracee->heap->prealloc_size);
-
 	/* Actually resizing.  */
 	set_sysnum(tracee, PR_mremap);
 	poke_reg(tracee, SYSARG_1 /* old_address */, tracee->heap->base - heap_offset);
@@ -154,7 +141,7 @@ word_t translate_brk_enter(Tracee *tracee)
 	poke_reg(tracee, SYSARG_4 /* flags       */, 0);
 	poke_reg(tracee, SYSARG_5 /* new_address */, 0);
 
-	return 0;
+	return;
 }
 
 /**
